import java.awt.*;
import java.awt.GraphicsDevice.*;
import java.awt.GraphicsEnvironment.*;
import java.awt.Graphics2D.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.SpinnerNumberModel.*;
import javax.swing.event.*;
import java.util.*;

public class WaveformWindow extends JFrame{
    public double [] TargetWaveform;
    public double [] ActualWaveform;  
    public int nSamples;
    public int MaxSamples = 1000;
    public int currentSample;
    public int zoom;
    public int weight;
    private String neuronName;
    private String neuronNumber;
    
    public WaveArea waveArea;
    public JPanel northPanel;
    public JPanel eastPanel;
    public JPanel westPanel;
    public JPanel southPanel;
    public JSpinner zoomSpinner;
    public JSpinner weightSpinner;
    public SpinnerNumberModel zoomSpinnerModel;
    public SpinnerNumberModel weightSpinnerModel;
    
    public double MeanSquareError()
    {
        double mse = 0;
        double diff = 0;
        for (int i=0; i<nSamples; i++)
        {
            diff = TargetWaveform[i] - ActualWaveform[i];
            diff = diff*diff;
            mse += diff /((double) nSamples);
        }
        return mse*((double)weight);
    }
    
    public WaveformWindow()
    {
        zoom = 1;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        //this.setBounds();
        this.setSize(500,150);
        zoomSpinner = new JSpinner();   
        weightSpinner = new JSpinner();   
        zoomSpinnerModel = new SpinnerNumberModel(1,1,10,1);
        weightSpinnerModel = new SpinnerNumberModel(0,0,10,1);
        zoomSpinner.setModel(zoomSpinnerModel);
        weightSpinner.setModel(weightSpinnerModel);
        zoomSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e)
            {
                zoom = zoomSpinnerModel.getNumber().intValue();
                repaint();
            }
        });
            
        weightSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e)
            {
                weight = weightSpinnerModel.getNumber().intValue();
                repaint();
            }
        });
              
        
        
        //zoomSpinnerModel.setValue(new Integer(1));
        
        //zoomSpinnerModel.setMinimum(new Integer(1));
        //zoomSpinnerModel.setMaximum(new Integer(10));
        
        waveArea = new WaveArea();
        waveArea.setWaveformWindow(this);
       // waveArea.setSize(1000,1000);
        northPanel = new JPanel();
        eastPanel = new JPanel();
        westPanel = new JPanel();
        southPanel = new JPanel();
        westPanel.setLayout(new FlowLayout());
        JLabel zoomLabel = new JLabel("Zoom");

        westPanel.add(zoomLabel);
        westPanel.add(zoomSpinner);
        westPanel.add(new JLabel("Error weight"));
        westPanel.add(weightSpinner);
        
              
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(waveArea,BorderLayout.CENTER);
        this.getContentPane().add(northPanel,BorderLayout.NORTH);
        this.getContentPane().add(southPanel,BorderLayout.SOUTH);
        this.getContentPane().add(eastPanel,BorderLayout.EAST);
        this.getContentPane().add(westPanel,BorderLayout.WEST);
        waveArea.setBackground(Color.WHITE);
        waveArea.setLocation(10,10);
        //WaveArea.setSize(this.getSize().width-20,this.getSize().height-20);
        waveArea.setLayout(null);
        waveArea.setVisible(true);
        currentSample = 0;
        nSamples = 0;
        System.out.println("waveform constructor fired!");        
        ActualWaveform = new double[MaxSamples];
        TargetWaveform = new double[MaxSamples];
        nSamples=100;
       /* for (int i=0; i<nSamples; i++)
        {
            ActualWaveform[i] = Math.sin(i/10.0);
            TargetWaveform[i] = 0.5*Math.sin(i/33.0+5.2);
        }*/
        
    }
    
    public void SetNextSample(double sample, boolean visible)
    {
        if (currentSample<MaxSamples)
        {
            ActualWaveform[currentSample] = sample;
            currentSample++;
        }
        if (visible)
        {
            this.repaint();
        }
    }
    public void ResetCurrentSample(boolean visible)
    {
        this.currentSample = 0;
        if (visible)
        {
            this.repaint();
        }
    }
    
    public void ShowWindow()
    {
        this.setVisible(true);
    }
    public void SetLength(int length){
    }
    public void SetNeuronName(String nname)
    {
        neuronName = nname;
        this.setTitle("Waveform - "+neuronNumber+" ("+neuronName+")");
    }
    public void SetNeuronNumber(int nnum)
    {
        neuronNumber = String.valueOf(nnum);
        this.setTitle("Waveform - "+neuronNumber+" ("+neuronName+")");
    }
    public void SetZoom(int zoom)
    {
        if (zoom<1) zoom = 1;
        if (zoom>3) zoom = 4;
    }
    public void paint()
    {
        
    }
}
